@@ -1,6 +1,6 @@
 import logging
 import subprocess
-from os import environ as env
+from os import environ
 
 from .py_op_exceptions import OPCmdFailedException, OPNotFoundException
 
@@ -15,16 +15,20 @@ TODO: Move other code that closely touches 'op' here
 
 
 class _OPCLIExecute:
-    NOT_SIGNED_IN_TEXT = "not currently signed in"
+    logger = logging.getLogger("_OPCLIExecute")
+    logger.setLevel(logging.INFO)
 
-    logging.basicConfig(format="%(message)s", level=logging.INFO)
-    logger = logging.getLogger()
+    def __new__(cls, *args, logger=None, **kwargs):
+        if logger:
+            print(f"setting logger on class: {cls}")
+            cls.set_logger(logger)
+        return super().__new__(cls)
     """
     Class for logging into and querying a 1Password account via the 'op' cli command.
     """
 
     @classmethod
-    def _run_raw(cls, argv, input_string=None, capture_stdout=False, ignore_error=False):
+    def _run_raw(cls, argv, input_string=None, capture_stdout=False, ignore_error=False, env=environ):
         stdout = subprocess.PIPE if capture_stdout else None
         if input_string:
             if isinstance(input_string, str):
@@ -42,19 +46,19 @@ class _OPCLIExecute:
                 _ran.check_returncode()
             except subprocess.CalledProcessError as err:
                 stderr_output = stderr.decode("utf-8").rstrip()
-                if env.get(LOG_OP_ERR_ENV_NAME) == "1":
+                if environ.get(LOG_OP_ERR_ENV_NAME) == "1":
                     cls.logger.error(stderr_output)
                 raise OPCmdFailedException(stderr_output, returncode) from err
 
         return (stdout, stderr, returncode)
 
     @classmethod
-    def _run(cls, argv, capture_stdout=False, input_string=None, decode=None):
+    def _run(cls, argv, capture_stdout=False, input_string=None, decode=None, env=environ):
         cls.logger.debug(f"Running: {argv.cmd_str()}")
         output = None
         try:
             output, _, _ = cls._run_raw(
-                argv, input_string=input_string, capture_stdout=capture_stdout)
+                argv, input_string=input_string, capture_stdout=capture_stdout, env=env)
             if decode:
                 output = output.decode(decode)
         except FileNotFoundError as err:
@@ -65,3 +69,12 @@ class _OPCLIExecute:
             raise OPNotFoundException(argv[0], err.errno) from err
 
         return output
+
+    @classmethod
+    def set_logger(cls, logger: logging.Logger):
+        cls.logger = logger
+        print("logger has been set")
+
+    @classmethod
+    def set_log_level(cls, log_level: int):
+        cls.logger.setLevel(log_level)
