@@ -1,8 +1,10 @@
 """
 Description: A module that maps methods to to `op` commands and subcommands
 """
+import enum
 import logging
-from os import environ as env
+from os import environ
+from typing import Union
 
 from ._op_cli_argv import _OPArgv
 from ._op_cli_config import OPCLIConfig
@@ -18,6 +20,7 @@ from .py_op_exceptions import (
     OPItemListException,
     OPNotSignedInException,
     OPSigninException,
+    OPUnknownAccountException,
     OPUserGetException,
     OPUserListException,
     OPVaultGetException,
@@ -25,6 +28,17 @@ from .py_op_exceptions import (
 )
 
 
+class ExistingAuthEnum(enum.IntEnum):
+    IGNORE = 0     # Don't check if there's an existing authentication, peform new one regardless
+    AVAILABLE = 1  # Check if there's an existing authentication, and if not, perform authentication
+    REQUIRED = 2   # Check if there's an existing authentication, failing otherwise
+
+
+EXISTING_AUTH_IGNORE = ExistingAuthEnum.IGNORE
+EXISTING_AUTH_AVAIL = ExistingAuthEnum.AVAILABLE
+EXISTING_AUTH_REQD = ExistingAuthEnum.REQUIRED
+
+
 class _OPCommandInterface(_OPCLIExecute):
     """
     A class that directly maps methods to `op` commands
@@ -32,16 +46,20 @@ class _OPCommandInterface(_OPCLIExecute):
     No convenience methods are provided.
     No responses are parsed.
     """
+    NOT_SIGNED_IN_TEXT = "not currently signed in"
+    NO_ACTIVE_SESSION_FOUND_TEXT = "no active session found for account"
+    NO_SESSION_TOKEN_FOUND_TEXT = "could not find session token for account"
+    ACCT_IS_NOT_SIGNED_IN_TEXT = "account is not signed in"
 
     OP_PATH = 'op'  # let subprocess find 'op' in the system path
 
     def __init__(self,
                  vault=None,
-                 account_shorthand=None,
+                 account=None,
                  password=None,
                  logger=None,
-                 op_path='op',
-                 use_existing_session=False,
+                 op_path=OP_PATH,
+                 existing_auth: ExistingAuthEnum = EXISTING_AUTH_IGNORE,
                  password_prompt=True):
         """
         Create an OP object. The 1Password sign-in happens during object instantiation.
@@ -59,50 +77,49 @@ class _OPCommandInterface(_OPCLIExecute):
             - OPNotFoundException if the 1Password command can't be found.
         """
         super().__init__()
-        self.vault = vault
         if not logger:
-            logging.basicConfig(format="%(message)s", level=logging.DEBUG)
-            logger = logging.getLogger()
+            logger = logging.getLogger(self.__class__.__name__)
+            logger.setLevel(logging.INFO)
+
+        self.vault = vault
         self.logger = logger
-        self._token = None
+
+        # Coerce existing_auth to an Enum in case it was passed in as a legacy bool
+        # False -> ExistingAuthFlag.NONE, True -> ExistingAuthFlag.AVAILABLE
+        existing_auth = ExistingAuthEnum(existing_auth)
+
         self.op_path = op_path
+        self._account_identifier = account
 
-        self._cli_version: OPCLIVersion = self._get_cli_version(op_path)
+        self._op_config: OPCLIConfig = None
+        self._cli_version: OPCLIVersion = None
+        self._account_list: OPAccountList = None
+        self._uses_bio: bool = False
+        self._sess_var: str = None
 
-        self.uses_bio = self.uses_biometric(
-            self.op_path, account_shorthand=account_shorthand)
-        op_config = OPCLIConfig()
-        if account_shorthand is None and not self.uses_bio:
-            account_shorthand = self._get_account_shorthand(op_config)
-            if account_shorthand is None:
-                raise OPNotSignedInException(
-                    "Account shorthand not provided and not found in 'op' config")
+        # gathering facts will attempt to set the above instance variables
+        # that got initialized to None or False
+        self._gather_facts()
 
-        self.account_shorthand = account_shorthand
-
-        sess_var_name = None
-
-        if account_shorthand and not self.uses_bio:
-            user_uuid = op_config.uuid_for_shorthand(account_shorthand)
-            sess_var_name = 'OP_SESSION_{}'.format(user_uuid)
-
-        if use_existing_session and sess_var_name:
-            self._token = self._verify_signin(sess_var_name)
-
-        if not self._token:
-            if not password and not password_prompt and not self.uses_bio:
-                # we don't have a token, weren't provided a password
-                # and were told not to let 'op' prompt for a password
-                raise OPNotSignedInException(
-                    "No existing session and no password provided.")
-            # We don't have a token but eitehr are have a password
-            # or op should be allowed to prompt for one
-            self._token = self._do_normal_signin(account_shorthand, password)
-
-        self._sess_var = sess_var_name
-        # export OP_SESSION_<signin_address>
-        if sess_var_name and self.token:
-            env[sess_var_name] = self.token
+        # So far everything above has been fairly lightweight, with no remote
+        # contact to the 1Password account.
+        # The next steps will attempt to talk to the 1Password account, and
+        # failing that, may attempt to authenticate to the 1Password account
+        account, token = self._new_or_existing_signin(
+            existing_auth, password, password_prompt)
+        self._signed_in_account: OPAccount = account
+        self._token = token
+        self.logger.debug(
+            f"Signed in as User ID: {self._signed_in_account.user_uuid}")
+        # export OP_SESSION_<use_id>
+        if self.token:
+            if not self._account_identifier:
+                # if we weren't provided an account identifier,
+                # we can now get it from the signed-in account object
+                # and compute the session environment variable name
+                self._account_identifier = account.user_uuid
+                self._sess_var = self._compute_session_var_name()
+            environ[self._sess_var] = self.token
 
     @property
     def token(self) -> str:
@@ -112,14 +129,32 @@ class _OPCommandInterface(_OPCLIExecute):
     def session_var(self) -> str:
         return self._sess_var
 
-    def _get_account_shorthand(self, config):
-        try:
-            account_shorthand = config['latest_signin']
-            self.logger.debug(
-                "Using account shorthand found in op config: {}".format(account_shorthand))
-        except KeyError:
-            account_shorthand = None
-        return account_shorthand
+    @classmethod
+    def uses_biometric(cls, op_path="op", encoding="utf-8", account_list=None):
+        uses_bio = True
+        # We can run 'op account list', which doesn't require talking (or authenticating)
+        # to the 1Password account
+        # if biometric is enabled, there will be no account shorthands in the output
+        # if there are account shorthands, biometric is not enabled
+        if account_list is None:
+            account_list = cls._get_account_list(op_path)
+        acct: OPAccount
+        for acct in account_list:
+            if not acct.shorthand:
+                continue
+            # There is at least one account_shorthand found in `op account list`
+            uses_bio = False
+            break
+        return uses_bio
+
+    def _gather_facts(self):
+        self._op_config = OPCLIConfig()
+        self._cli_version: OPCLIVersion = self._get_cli_version(self.op_path)
+        self._account_list = self._get_account_list(self.op_path)
+        self._uses_bio = self.uses_biometric(
+            op_path=self.op_path, account_list=self._account_list)
+        self._account_identifier = self._normalize_account_id()
+        self._sess_var = self._compute_session_var_name()
 
     def _get_cli_version(self, op_path):
         argv = _OPArgv.cli_version_argv(op_path)
@@ -128,63 +163,168 @@ class _OPCommandInterface(_OPCLIExecute):
         cli_version = OPCLIVersion(output)
         return cli_version
 
-    def _verify_signin(self, sess_var_name):
-        # Need to get existing token if we're already signed in
-        token = env.get(sess_var_name)
+    @classmethod
+    def _get_account_list(cls, op_path) -> OPAccountList:
+        account_list_json = cls._signed_in_accounts(op_path)
+        account_list = OPAccountList(account_list_json)
+        return account_list
 
-        if token:
-            # if there's no token, no need to sign in
-            argv = _OPArgv.get_verify_signin_argv(self.op_path)
-            try:
-                self._run(argv, capture_stdout=True)
-            except OPCmdFailedException as opfe:
-                # scrape error message about not being signed in
-                # invalidate token if we're not signed in
-                if self.NOT_SIGNED_IN_TEXT in opfe.err_output:
-                    token = None
-                else:
-                    # there was a different error so raise the exception
-                    raise opfe
+    def _normalize_account_id(self):
+        # we need to turn whatever we were given into a User ID
+        # we could have been given any of the following for account ID:
+        # - None
+        # - account shorthand
+        # - account URL
+        # - account UUID
+        # - user UUID
+        # see: 'op --help' for '--account' global flag
+        user_uuid = None
+        if not self._uses_bio:
+            if self._account_identifier:
+                # if we were given an specific account look up user ID for that
+                user_uuid = self._op_config.uuid_for_account(
+                    self._account_identifier)
+            else:
+                # else try to look up User ID for latest sign-in
+                # (may come back None)
+                user_uuid = self._op_config.latest_signin_uuid
+        elif self._account_identifier:
+            user_uuid = self._account_list.user_id_for_account_identifier(
+                self._account_identifier)
+
+        if self._account_identifier and not user_uuid:
+            raise OPUnknownAccountException(
+                f"No account found for identifier: {self._account_identifier}")
+
+        return user_uuid
+
+    def _compute_session_var_name(self):
+        sess_var_name = self._sess_var
+        # normalize account identifier before calling this
+        user_uuid = self._account_identifier
+
+        # we can only use a session variable if:
+        #   - biometric isn't enabled, and
+        #   - we have an account shorthand
+
+        if user_uuid and not self._uses_bio:
+            sess_var_name = 'OP_SESSION_{}'.format(user_uuid)
+        return sess_var_name
+
+    def _new_or_existing_signin(self, existing_auth: ExistingAuthEnum, password: str, password_prompt: bool):
+        token = None
+        account = None
+
+        # Don't attempt to verify sign-in unless caller told us to
+        # otherwise it will trigger a prompt, either GUI biometric prompt,
+        # or interactive console prompt
+        if existing_auth in [EXISTING_AUTH_AVAIL, EXISTING_AUTH_REQD]:
+            account = self._verify_signin()
+
+        if account:
+            token = self._get_existing_token(account)
+        else:
+            if existing_auth == EXISTING_AUTH_REQD:
+                # we were told to only use existing authentication but verificaiton failed
+                # this is a hard error
+                raise OPNotSignedInException(
+                    "Existing authentication specified as required, but could not be verified.")
+
+            # we couldn't verify being signed in (or weren't told to try)
+            # let's try a normal sign-in
+            # _do_normal_signin() will raise OPNotSignedInException if
+            # _uses_bio is false, no password given, and password prompt not allowed
+            token = self._do_normal_signin(password, password_prompt)
+            account = self._verify_signin(token=token)
+        return (account, token)
+
+    def _verify_signin(self, token=None):
+        account: OPAccount = None
+        if token and self._sess_var:
+            # we need to pass an environment dictionary to subprocess.run() that contains
+            # the session token we're trying to verify
+            #
+            # make a copy of environment rather than modifying actual env
+            # in order to verify login
+            # we'll offically set it once we know it works
+            env = dict(environ)
+            env[self._sess_var] = token
+        else:
+            # we don't have a token to verify
+            # so no need to modify or copy the environment
+            # we can use it as-is
+            env = environ
+
+        # this step actually talks to the 1Password account
+        # it uses "op whoami" which is a very non-intrusive
+        # query that will fail without authentication
+        argv = _OPArgv.whoami_argv(
+            self.op_path, account=self._account_identifier)
+        try:
+            account_json = self._run(
+                argv, capture_stdout=True, decode="utf-8", env=env)
+            account = OPAccount(account_json)
+        except OPCmdFailedException as opfe:
+            # scrape error message about not being signed in
+
+            fragments = [self.NO_ACTIVE_SESSION_FOUND_TEXT,
+                         self.NOT_SIGNED_IN_TEXT,
+                         self.NO_SESSION_TOKEN_FOUND_TEXT,
+                         self.ACCT_IS_NOT_SIGNED_IN_TEXT]
+            unknown_err = True
+            for frag in fragments:
+                if frag in opfe.err_output:
+                    unknown_err = False
+                    break
+            # there was a different error so raise the exception
+            if unknown_err:  # pragma: no cover
+                raise opfe
+
+        return account
+
+    def _do_normal_signin(self, password: str, password_prompt: bool) -> Union[str, None]:
+        # normalize empty string to None, otherwise use password as given
+        password = None if password == "" else password
+
+        # there are three things that can be used for a *new* authentication:
+        # - biometric
+        # - password
+        # - allow 'op' to interactively prompt
+        # NOTE: we need to keep this check as close to the call to _run_signin() as possible
+        #       We can't tell 'op' not to prompt. We just have to not run it if we're not prompting
+        if not self._uses_bio and not password and not password_prompt:
+            # - we weren't provided a password, and
+            # - we were told not to let 'op' prompt for a password, and
+            # - biometric is not enabled
+            # so we can't sign in
+            raise OPNotSignedInException(
+                "No existing session and no password provided.")
+        signin_argv = _OPArgv.normal_signin_argv(
+            self.op_path, account=self._account_identifier)
+
+        token = self._run_signin(signin_argv, password=password)
+        if self._uses_bio:
+            # Set to None vs empty string
+            token = None
 
         return token
 
-    def _do_normal_signin(self, account_shorthand, password):
-        signin_argv = _OPArgv.normal_signin_argv(
-            self.op_path, account=account_shorthand)
-
-        token = self._run_signin(signin_argv, password=password).rstrip()
-        return token.decode()
+    def _get_existing_token(self, account: OPAccount):
+        if self._sess_var:
+            token = environ.get(self._sess_var)
+        else:
+            token = None
+        return token
 
     def _run_signin(self, argv, password=None):
         try:
             output = self._run(argv, capture_stdout=True,
-                               input_string=password)
+                               input_string=password, decode="utf-8")
         except OPCmdFailedException as opfe:
             raise OPSigninException.from_opexception(opfe) from opfe
 
         return output
 
-    @classmethod
-    def uses_biometric(cls, op_path="op", account_shorthand=None, encoding="utf-8"):
-        uses_bio = True
-        account_list_argv = cls._account_list_argv(
-            op_path=op_path, encoding=encoding)
-        account_list_json = cls._run(
-            account_list_argv, capture_stdout=True, decode=encoding)
-        account_list = OPAccountList(account_list_json)
-        acct: OPAccount
-        for acct in account_list:
-            if not acct.shorthand:
-                continue
-            if account_shorthand:
-                if acct.shorthand != account_shorthand:
-                    continue
-            # There is at least one account_shorthand found in `op account list`
-            # and if we were given a specific shorthand to use, it matches
-            uses_bio = False
-            break
-        return uses_bio
-
     @classmethod
     def _account_list_argv(cls, op_path="op", encoding="utf-8"):
         argv = _OPArgv.account_list_argv(op_path, encoding=encoding)
@@ -242,11 +382,6 @@ class _OPCommandInterface(_OPCLIExecute):
             self.op_path, group_name_or_id=group_name_or_id, user_name_or_id=user_name_or_id)
         return vault_list_argv
 
-    def _cli_version_argv(self):
-        # Specifically for use by mock_op response-generator
-        cli_version_argv = _OPArgv.cli_version_argv(self.op_path)
-        return cli_version_argv
-
     def _item_get(self, item_name_or_id, vault=None, fields=None, decode="utf-8"):
         get_item_argv = self._item_get_argv(
             item_name_or_id, vault=vault, fields=fields)
@@ -290,7 +425,7 @@ class _OPCommandInterface(_OPCLIExecute):
         except OPCmdFailedException as ocfe:
             raise OPDocumentGetException.from_opexception(ocfe) from ocfe
 
-        if self._cli_version <= DOCUMENT_BYTES_BUG_VERSION:
+        if self._cli_version <= DOCUMENT_BYTES_BUG_VERSION:  # pragma: no cover
             # op v2.x appends an erroneous \x0a ('\n') byte to document bytes
             # trim it off if its present
             if document_bytes[-1] == 0x0a:
@@ -299,10 +434,11 @@ class _OPCommandInterface(_OPCLIExecute):
             print(self._cli_version)
         return document_bytes
 
-    def _signed_in_accounts(self, decode="utf-8"):
-        account_list_argv = self._account_list_argv(op_path=self.op_path)
-        output = self._run(account_list_argv,
-                           capture_stdout=True, decode=decode)
+    @classmethod
+    def _signed_in_accounts(cls, op_path, decode="utf-8"):
+        account_list_argv = cls._account_list_argv(op_path)
+        output = cls._run(account_list_argv,
+                          capture_stdout=True, decode=decode)
         return output
 
     def _user_get(self, user_name_or_id: str, decode: str = "utf-8") -> str:
@@ -367,16 +503,16 @@ class _OPCommandInterface(_OPCLIExecute):
             raise OPVaultListException.from_opexception(ocfe)
         return output
 
-    def _signout(self, account, session, forget=False):
-        if forget and self.uses_bio:
+    def _signout(self, account, session, forget=False):   # pragma: no cover
+        if forget and self._uses_bio:
             self.logger.warn(
                 "Biometric is enabled. 'forget' operation will have no effect.")
         argv = _OPArgv.signout_argv(
-            self.op_path, account, session, forget=forget, uses_bio=self.uses_bio)
+            self.op_path, account, session, forget=forget, uses_bio=self._uses_bio)
         self._run(argv)
 
     @classmethod
-    def _forget(cls, account: str, op_path=None):
+    def _forget(cls, account: str, op_path=None):   # pragma: no cover
         if not op_path:
             op_path = cls.OP_PATH
         argv = _OPArgv.forget_argv(op_path, account)
