@@ -1,5 +1,5 @@
 from abc import abstractmethod
-from typing import List
+from typing import List, Union
 
 from ._item_descriptor_base import OPAbstractItemDescriptor
 from .item_section import OPItemField, OPSection
@@ -64,11 +64,29 @@ class OPAbstractItem(OPAbstractItemDescriptor):
                 f"Field not found with ID: {field_id}")
         return field
 
+    def fields_by_label(self, field_label: str) -> List[OPItemField]:
+        fields = []
+        f: OPItemField
+        for _, f in self._field_map.items():
+            if f.label == field_label:
+                fields.append(f)
+        return fields
+
+    def first_field_by_label(self, field_label: str) -> OPItemField:
+        fields = self.fields_by_label(field_label)
+        f = fields[0]
+        return f
+
     def field_value_by_id(self, field_id):
         field = self.field_by_id(field_id)
         value = field.value
         return value
 
+    def field_reference_by_id(self, field_id) -> Union[str, None]:
+        field = self.field_by_id(field_id)
+        ref = field.reference
+        return ref
+
     def _field_value_from_section(self, section: OPSection, field_label: str):
         section_field: OPItemField = section.fields_by_label(field_label)[0]
         value = section_field.value
