@@ -2,6 +2,7 @@ import logging
 from os import environ as env
 
 from ._py_op_commands import _OPCommandInterface
+from ._py_op_deprecation import deprecated_kwargs
 from .account import OPAccountList
 from .op_items._item_list import OPItemList
 from .op_items._op_item_type_registry import OPItemFactory
@@ -29,46 +30,41 @@ class OP(_OPCommandInterface):
     """
     Class for logging into and querying a 1Password account via the 'op' cli command.
     """
-
+    @deprecated_kwargs(use_existing_session='existing_auth',
+                       account_shorthand='account')
     def __init__(self,
                  vault: str = None,
+                 account: str = None,
                  account_shorthand: str = None,
                  password: str = None,
                  logger: logging.Logger = None,
                  op_path: str = 'op',
+                 existing_auth: bool = False,
                  use_existing_session: bool = False,
                  password_prompt: bool = True):
         """
-        Create an OP object. The 1Password sign-in happens during object instantiation.
-
-        Authentication is attempted in the following order:
-        - If use_existing_session is True and a session token is found
-          in an OP_SESSION_<user UUID> environment variable, that session is verified
-          and used
-        - If biometric authentication is enabled, it will be used
-        - If 'password' is provided, it will be provided to the 'op' command
-        - If no password is provided, but password_prompt is True, 'op' will be allowed to prompt for a password
-        - If authentication is attempted and failed (password or biometric), OPSigninException is raised
-        - If no authentication is attempted and (no password, no password prompt, biometric not enabled), and no
-          valid session is found, OPNotSignedInException is raised
+        Create an OP object. The 1Password (non-initial) sign-in happens during object instantiation.
 
         Parameters
         ----------
         vault : str, optional
             If set, this becomes the default argument to the --vault flag
             for future queries.
-        account_shorthand : str, optional
-            The shorthand name for the account on this device. You may choose this
-            during initial signin, otherwise 1Password converts it from your account
-            address. See 'op signin --help' for more information.
+        account : str, optional
+            May be any account identifier accepted by the op --account flag:
+            account shorthand, sign-in address, account ID, or user ID
         password : str, optional
             If provided, the password will be piped to the 'op' command over stdin
         logger : logging.Logger
             A logging object. If not provided a basic logger is created and used
         op_path : str, optional
             Optional path to the `op` command, if it's not at the default location
-        use_existing_session : bool
-            Whether an existing login session should be used if possible
+        existing_auth : ExistingAuthEnum
+            Whether existing authentication should be must be used, used if possible, or ignored
+            Valid values:
+              - EXISTING_AUTH_AVAIL: Use existing authentication if available
+              - EXISTING_AUTH_IGNORE: Reauthenticate, ignoring any existing authentication
+              - EXISTING_AUTH_REQD: Use existing authentication, failing if there isn't a valid session available
         password_prompt : bool
             Whether an interactive password prompt on the console should be presented if necessary
 
@@ -86,11 +82,11 @@ class OP(_OPCommandInterface):
             If the 1Password command can't be found
         """
         super().__init__(vault=vault,
-                         account_shorthand=account_shorthand,
+                         account=account,
                          password=password,
                          logger=logger,
                          op_path=op_path,
-                         use_existing_session=use_existing_session,
+                         existing_auth=existing_auth,
                          password_prompt=password_prompt)
 
     def item_get(self, item_name_or_id, vault=None) -> OPAbstractItem:
@@ -397,11 +393,11 @@ class OP(_OPCommandInterface):
         return item_list
 
     def signed_in_accounts(self, decode="utf-8") -> OPAccountList:
-        account_list_json = super()._signed_in_accounts(decode)
+        account_list_json = super()._signed_in_accounts(self.op_path, decode=decode)
         account_list = OPAccountList(account_list_json)
         return account_list
 
-    def signout(self, forget=False):
+    def signout(self, forget=False):  # pragma: no coverage
         """
         Sign out of the account used to create this OP instance
         This is equivalent to the command 'op signout'
@@ -422,9 +418,9 @@ class OP(_OPCommandInterface):
         -------
         None
         """
-        account = self.account_shorthand
+        account = self._account_identifier
         token = self.token
-        if not token and not self.uses_bio:
+        if not token and not self._uses_bio:
             return
 
         try:
@@ -437,7 +433,7 @@ class OP(_OPCommandInterface):
         self._sanitize()
 
     @classmethod
-    def forget(cls, account: str, op_path=None):
+    def forget(cls, account: str, op_path=None):  # pragma: no coverage
         """
         Remove details for the specified account from this device
         This is equivalent to the command 'op forget <account>'
@@ -469,10 +465,10 @@ class OP(_OPCommandInterface):
         except OPCmdFailedException as ocfe:
             raise OPForgetException.from_opexception(ocfe) from ocfe
 
-    def _sanitize(self):
+    def _sanitize(self):  # pragma: no coverage
         self._token = None
-        sess_var_name = 'OP_SESSION_{}'.format(self.account_shorthand)
-        try:
-            env.pop(sess_var_name)
-        except KeyError:
-            pass
+        if self._sess_var:
+            try:
+                env.pop(self._sess_var)
+            except KeyError:
+                pass
